/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <qprinter.h> // has to be first

#include <iostream>
#include <cstdlib>
#include <ctime>
#include <cassert>

#include <qlayout.h>
#include <qwidgetstack.h>
#include <qsplitter.h>
#include <qpushbutton.h>
#include <qprintdialog.h>
#include <qcollection.h>
#include <qkeycode.h>
#include <kaction.h>
#include <qmime.h>
#include <qtoolbutton.h>
#include <qtimer.h>

#include <kdialogbase.h>
#include <kdebug.h>
#include <kapp.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kstdaccel.h>
#include <kstdaction.h>
#include <kglobal.h>
#include <kmessagebox.h>
#include <kformulaedit.h>

#include <dcopclient.h>
#include <dcopref.h>

#include <kparts/event.h>
#include <koPartSelectDia.h>
#include <koQueryTrader.h>
#include <koPageLayoutDia.h>

#include "kivio_dlg_pageshow.h"
#include "kivio_factory.h"
#include "kivio_map.h"
#include "kivio_page.h"
#include "kivio_doc.h"
#include "kivio_shell.h"
#include "kivio_canvas.h"
#include "kivio_stencil_spawner.h"
#include "kivio_tabbar.h"
#include "kivio_view.h"
#include "kivio_ruler.h"
#include "kivio_zoomaction.h"
#include "toolbarseparator.h"
#include "coloractions.h"
#include "kivio_lineendsaction.h"
#include "tkfloatspinboxaction.h"
#include "tk2floatspinboxaction.h"
#include "tkunitsaction.h"
#include "tooldockmanager.h"
#include "tooldockbase.h"

#include "stencilbardockmanager.h"
#include "add_spawner_set_dlg.h"
#include "kivio_common.h"
#include "kivio_painter.h"
#include "kivio_stencil.h"
#include "kivio_stencil_spawner_set.h"
#include "kivio_screen_painter.h"
#include "kivio_ps_printer.h"


#include "tool_controller.h"

#include "handler.h"

#include "kivio_stackbar.h"
#include "kivio_icon_view.h"

#include "kivio_paragraph_action.h"

#include <qiconview.h>




KivioView::KivioView( QWidget *_parent, const char *_name, KivioDoc* doc )
: KoView( doc, _parent, _name )
{
  m_pDoc = doc;
  bool isModified = doc->isModified();
  m_pTools = new ToolController(this);
  m_pDockManager = new StencilBarDockManager(this);
  m_pDockManager->setDoc( doc );

  // QGridLayout for the entire view
  QGridLayout *viewGrid = new QGridLayout(this);
  viewGrid->addWidget(m_pDockManager, 0, 0);

  // Load any already-loaded stencils into the stencil dock
  KivioStencilSpawnerSet *pSet;
  pSet = m_pDoc->spawnerSets()->first();
  while( pSet )
  {
    addSpawnerToStackBar( pSet );
    pSet = m_pDoc->spawnerSets()->next();
  }

  // A widget to hold the entire right side (old view)
  QWidget *pRightSide = new QWidget(m_pDockManager);
  m_pDockManager->setView(pRightSide);

  // Split tabbar and Horz. Scroll Bar
  QSplitter* tabSplit = new QSplitter(pRightSide);

  // Tab Bar
  m_pTabBar = new KivioTabBar(tabSplit,this);
  connect( m_pTabBar,
           SIGNAL(tabChanged(const QString&)),
           SLOT( changePage(const QString&)));

  // Scroll Bar
  QScrollBar* vertScrollBar = new QScrollBar(QScrollBar::Vertical,pRightSide);
  QScrollBar* horzScrollBar = new QScrollBar(QScrollBar::Horizontal,tabSplit);
  // Ruler's
  KivioRuler* vRuler = new KivioRuler(KivioRuler::Vertical,UnitMillimeter,pRightSide);
  KivioRuler* hRuler = new KivioRuler(KivioRuler::Horizontal,UnitMillimeter,pRightSide);

  // Tab Bar Button
  m_pTabBarFirst = newIconButton("arrow_first", false, pRightSide);
  connect( m_pTabBarFirst,
           SIGNAL(clicked()),
           m_pTabBar,
           SLOT(scrollFirst()));

  m_pTabBarLeft = newIconButton("arrow_back", false, pRightSide);
  connect( m_pTabBarLeft,
           SIGNAL(clicked()),
           m_pTabBar,
           SLOT(scrollLeft()));

  m_pTabBarRight = newIconButton("arrow_forward", false, pRightSide);
  connect( m_pTabBarRight,
           SIGNAL(clicked()),
           m_pTabBar,
           SLOT(scrollRight()));

  m_pTabBarLast = newIconButton("arrow_last", false, pRightSide);
  connect( m_pTabBarLast,
           SIGNAL(clicked()),
           m_pTabBar,
           SLOT(scrollLast()));

  QHBoxLayout* tabLayout = new QHBoxLayout();
  tabLayout->addWidget(m_pTabBarLast);
  tabLayout->addWidget(m_pTabBarLeft);
  tabLayout->addWidget(m_pTabBarRight);
  tabLayout->addWidget(m_pTabBarFirst);
  tabLayout->addWidget(tabSplit);

  // The widget on which we display the page
  QWidgetStack* canvasBase = new QWidgetStack(pRightSide);
  m_pCanvas = new KivioCanvas(canvasBase,this,doc,m_pTools,vertScrollBar,horzScrollBar,vRuler,hRuler);
  canvasBase->addWidget(m_pCanvas,0);
  canvasBase->raiseWidget(m_pCanvas);
  m_pCanvas->setFocusPolicy(QWidget::StrongFocus);
  connect( m_pCanvas,
           SIGNAL(zoomChanges(int)),
           SLOT(canvasZoomChanged(int)));

  QGridLayout* layout = new QGridLayout(pRightSide);
  layout->addWidget(hRuler,0,1);
  layout->addWidget(vRuler,1,0);
  layout->addWidget(canvasBase,1,1);
  layout->addMultiCellLayout(tabLayout,2,2,0,1);
  layout->addMultiCellWidget(vertScrollBar,0,1,2,2);

  layout->setRowStretch(1,1);

  QWidget::setFocusPolicy( QWidget::StrongFocus );
  setFocusProxy( m_pCanvas );

  connect( this, SIGNAL( invalidated() ), m_pCanvas, SLOT( update() ) );
  connect( this, SIGNAL( regionInvalidated( const QRegion&, bool ) ), m_pCanvas, SLOT( repaint( const QRegion&, bool ) ) );

  m_pToolDock = new ToolDockManager(canvasBase);

  setInstance(KivioFactory::global());
  setXMLFile("kivio.rc");
  setupActions();

  KivioPage* page;
  for ( page = m_pDoc->map()->firstPage(); page; page = m_pDoc->map()->nextPage() )
    addPage(page);

  setActivePage(m_pDoc->map()->firstPage());

  connect( m_pDoc, SIGNAL( sig_selectionChanged() ), SLOT( updateToolBars() ) );
  connect( m_pDoc, SIGNAL( sig_addPage( KivioPage* ) ), SLOT( slotAddPage( KivioPage* ) ) );
  connect( m_pDoc, SIGNAL( sig_addSpawnerSet( KivioStencilSpawnerSet* ) ), SLOT( addSpawnerToStackBar(KivioStencilSpawnerSet*) ) );

  
  initActions();

  m_pCanvas->centerPage();
  m_pDoc->setModified(isModified);
}

KivioView::~KivioView()
{
}

void KivioView::setupActions()
{

  (void) new ToolBarSeparator(actionCollection(),"---");

  m_unitAct = new TKUnitsAction(actionCollection(),"units");
  connect( m_unitAct, SIGNAL(activated(int)), m_pDoc, SLOT(setUnits(int)) );
  connect( m_pDoc, SIGNAL(unitsChanged(int)), m_unitAct, SLOT(setCurrentItem(int)) );
  
  
  AddSpawnerSetAction* addSpSet =  new AddSpawnerSetAction( i18n("Add Stencil Set"), "open_stencilset", 0, actionCollection(), "addStencilSet" );
  connect(addSpSet,SIGNAL(activated(const QString&)),SLOT(addStencilSet(const QString&)));

  (void) new KAction( i18n("Cut"), "cut_stencil", CTRL+Key_X, this, SLOT(cutStencil()), actionCollection(), "cutStencil" );
  (void) new KAction( i18n("Copy"), "copy_stencil", CTRL+Key_C, this, SLOT(copyStencil()), actionCollection(), "copyStencil" );
  (void) new KAction( i18n("Paste"), "paste_stencil", CTRL+Key_V, this, SLOT(pasteStencil()), actionCollection(), "pasteStencil" );

  (void) new KAction( i18n("Select All"), "select_all_stencils", CTRL+Key_A, this, SLOT(selectAllStencils()), actionCollection(), "selectAllStencils" );
  (void) new KAction( i18n("Select None"), "unselect_all_stencils", CTRL+SHIFT+Key_A, this, SLOT(unselectAllStencils()), actionCollection(), "unselectAllStencils" );

  (void) new KAction( i18n("Group Selected Stencils"), "group_stencils", CTRL+Key_G, this, SLOT(groupStencils()), actionCollection(), "groupStencils" );
  (void) new KAction( i18n("Ungroup Selected Stencils"), "ungroup_stencils", CTRL+SHIFT+Key_G, this, SLOT(ungroupStencils()), actionCollection(), "ungroupStencils" );

  (void) new KAction( i18n("Bring To Front"), "bring_stencil_to_front", 0, this, SLOT(bringStencilToFront()), actionCollection(), "bringStencilToFront" );
  (void) new KAction( i18n("Send To Back"), "send_stencil_to_back", 0, this, SLOT(sendStencilToBack()), actionCollection(), "sendStencilToBack" );

  /* Create the fg color button */
  m_setFGColor = new SelectColorAction( i18n("Set Foreground Color"), SelectColorAction::LineColor, actionCollection(), "setFGColor" );
  connect(m_setFGColor,SIGNAL(activated()),SLOT(setFGColor()));
  m_setBGColor = new SelectColorAction( i18n("Set Background Color"), SelectColorAction::FillColor, actionCollection(), "setBGColor" );
  connect(m_setBGColor,SIGNAL(activated()),SLOT(setBGColor()));


  // Text bar
  m_setFontFamily = new KFontAction( i18n( "Set Font Family" ), 0, actionCollection(), "setFontFamily" );
  connect( m_setFontFamily, SIGNAL(activated(const QString&)), SLOT(setFontFamily(const QString&)) );

  m_setFontSize = new KFontSizeAction( i18n( "Set Font Size" ), 0, actionCollection(), "setFontSize" );
  connect( m_setFontSize, SIGNAL(activated(const QString&)), SLOT(setFontSize(const QString&)) );

  m_setTextColor = new SelectColorAction( i18n("Set Text Color"), SelectColorAction::TextColor, actionCollection(), "setTextColor" );
  connect( m_setTextColor, SIGNAL(activated()), SLOT(setTextColor()) );

  m_setBold = new KToggleAction( i18n("Toggle Bold Text"), "font_bold", 0, actionCollection(), "setFontBold" );
  connect( m_setBold, SIGNAL(toggled(bool)), SLOT(toggleFontBold(bool)) );

  m_setItalics = new KToggleAction( i18n("Toggle Italics Text"), "font_italic", 0, actionCollection(), "setFontItalics" );
  connect( m_setItalics, SIGNAL(toggled(bool)), SLOT(toggleFontItalics(bool)) );

  m_setUnderline = new KToggleAction( i18n("Toggle Underline Text"), "font_under", 0, actionCollection(), "setFontUnderline" );
  connect( m_setUnderline, SIGNAL(toggled(bool)), SLOT(toggleFontUnderline(bool)));

  m_setHTextAlignment = new KivioParagraphAction( false, actionCollection(), "setHTextAlignment" );
  m_setVTextAlignment = new KivioParagraphAction( true, actionCollection(), "setVTextAlignment" );
  connect( m_setHTextAlignment, SIGNAL(activated(int)), SLOT(setHParaAlign(int)) );
  connect( m_setVTextAlignment, SIGNAL(activated(int)), SLOT(setVParaAlign(int)) );

  m_setLineWidth = new TKUFloatSpinBoxAction(  i18n("Set Line Width"), "linewidth", 0, actionCollection(), "setLineWidth" );
  m_setLineWidth->setIconMode(TK::IconOnly);
  m_setLineWidth->setDecimals(2);
  m_setLineWidth->setMinValue(0.0);
  m_setLineWidth->setLineStep(1.0);
  connect( m_setLineWidth, SIGNAL(activated()), this, SLOT(setLineWidth()) );
  connect( m_unitAct, SIGNAL(activated(int)), m_setLineWidth, SLOT(setUnit(int)) );

  m_paperLayout = new KAction( i18n("Paper Layout..."), 0, this, SLOT(paperLayoutDlg()), actionCollection(), "paperLayout" );
  m_insertPage = new KAction( i18n("Insert Page"),"insertpage", 0, this, SLOT(insertPage()), actionCollection(), "insertPage" );
  m_removePage = new KAction( i18n("Remove Page"), "delete_page",0,this, SLOT(removePage()), actionCollection(), "removePage" );
  m_showPage = new KAction( i18n("Show Page..."),0 ,this,SLOT(showPage()), actionCollection(), "showPage" );
  m_hidePage = new KAction( i18n("Hide Page"),0 ,this,SLOT(hidePage()), actionCollection(), "hidePage" );

  m_showPageBorders = new KToggleAction( i18n("Show page borders"), "view_pageborder", CTRL+Key_B, actionCollection(), "showPageBorders" );
  connect( m_showPageBorders,
           SIGNAL(toggled(bool)),
           SLOT(togglePageBorders(bool)));

  m_showPageMargins = new KToggleAction( i18n("Show page margins"), "view_margins", 0, actionCollection(), "showPageMargins" );
  connect( m_showPageMargins,
           SIGNAL(toggled(bool)),
           SLOT(togglePageMargins(bool)));

  m_showRulers = new KToggleAction( i18n("Show rulers"), "view_ruler", 0, actionCollection(), "showRulers" );
  connect( m_showRulers,
           SIGNAL(toggled(bool)),
           SLOT(toggleShowRulers(bool)));

  m_showGrid = new KToggleAction( i18n("Show grid"), "view_grid", 0, actionCollection(), "showGrid" );
  connect( m_showGrid,
           SIGNAL(toggled(bool)),
           SLOT(toggleShowGrid(bool)));

  m_viewZoom = new ZoomAction(actionCollection(),"viewZoom");
  m_viewZoom->setIcon("kivio_zoom");
  connect( m_viewZoom,
           SIGNAL(zoomActivated(int)),	
	         SLOT(viewZoom(int)));

  m_setEndArrow = new LineEndsAction( false, actionCollection(), "endArrowHead" );
  m_setStartArrow = new LineEndsAction( true, actionCollection(), "startArrowHead" );

  connect( m_setEndArrow, SIGNAL(activated(int)), SLOT(slotSetEndArrow(int)));
  connect( m_setStartArrow, SIGNAL(activated(int)), SLOT(slotSetStartArrow(int)));

  m_setEndArrowSize = new TKSizeAction(actionCollection(), "endArrowSize");
  m_setStartArrowSize = new TKSizeAction(actionCollection(), "startArrowSize");

  connect( m_setEndArrowSize, SIGNAL(activated()), SLOT(slotSetEndArrowSize()));
  connect( m_setStartArrowSize, SIGNAL(activated()), SLOT(slotSetStartArrowSize()));

  m_setStencilSize = new TKSizeAction(actionCollection(), "stencilSize");
  m_setStencilPos = new TKPositionAction(actionCollection(), "stencilPos");
  connect( m_setStencilSize, SIGNAL(activated()), SLOT(slotStencilSize()));
  connect( m_setStencilPos, SIGNAL(activated()), SLOT(slotStencilPos()));

  connect( m_unitAct, SIGNAL(activated(int)), m_setStencilSize, SLOT(setUnit(int)) );
  connect( m_unitAct, SIGNAL(activated(int)), m_setStencilPos, SLOT(setUnit(int)) );
}

void KivioView::initActions()
{
  togglePageBorders(true);
  togglePageMargins(true);
  toggleShowRulers(true);
  toggleShowGrid(false);
  viewZoom(m_pCanvas->zoom());
  m_unitAct->setCurrentItem(m_pDoc->units());
  m_unitAct->activate(m_pDoc->units());
}

void KivioView::viewGUIActivated( bool active )
{
  if ( active )
    m_pTools->activateView(this);
}

QButton* KivioView::newIconButton( const char* file, bool kbutton, QWidget* parent )
{
  if (!parent)
    parent = this;

  QPixmap *pixmap = new QPixmap(BarIcon(file));

  QButton *pb;
  if (!kbutton)
    pb = new QPushButton(parent);
  else
    pb = new QToolButton(parent);

  if (pixmap)
    pb->setPixmap(*pixmap);

  pb->setFixedSize(16,16);
  return pb;
}

void KivioView::updateReadWrite( bool readwrite )
{
  QValueList<KAction*> actions = actionCollection()->actions();
  QValueList<KAction*>::ConstIterator aIt = actions.begin();
  QValueList<KAction*>::ConstIterator aEnd = actions.end();
  for (; aIt != aEnd; ++aIt )
    (*aIt)->setEnabled( readwrite );

  m_showPage->setEnabled( true );
  m_hidePage->setEnabled( true );
}


void KivioView::addPage( KivioPage* page )
{
  if( !page->isHidden() ) {
    m_pTabBar->addTab(page->pageName());
    setActivePage(page);
  } else {
    m_pTabBar->addHiddenTab(page->pageName());
  }

  // Connect some signals
  connect( page,
           SIGNAL(sig_updateView(KivioPage*)),
           SLOT(slotUpdateView(KivioPage*)));


/*
  connect( page, SIGNAL( sig_updateView( KivioPage *, const QRect& ) ),
        SLOT( slotUpdateView( KivioPage*, const QRect& ) ) );
  connect( page, SIGNAL( sig_unselect( KivioPage *, const QRect& ) ),
        SLOT( slotUnselect( KivioPage *, const QRect& ) ) );
  connect( page, SIGNAL( sig_updateHBorder( KivioPage * ) ),
        SLOT( slotUpdateHBorder( KivioPage * ) ) );
  connect( page, SIGNAL( sig_updateVBorder( KivioPage * ) ),
        SLOT( slotUpdateVBorder( KivioPage * ) ) );
  connect( page, SIGNAL( sig_changeSelection( KivioPage *, const QRect &, const QRect & ) ),
        SLOT( slotChangeSelection( KivioPage *, const QRect &, const QRect & ) ) );
  connect( page, SIGNAL( sig_changeChooseSelection( KivioPage *, const QRect &, const QRect & ) ),
        SLOT( slotChangeChooseSelection( KivioPage *, const QRect &, const QRect & ) ) );
  connect( page, SIGNAL( sig_nameChanged( KivioPage*, const QString& ) ),
        this, SLOT( slotPageRenamed( KivioPage*, const QString& ) ) );
  // ########### Why do these signals not send a pointer to the page?
  // This will lead to bugs.
  connect( page, SIGNAL( sig_updateChildGeometry( KivioChild* ) ),
        SLOT( slotUpdateChildGeometry( KivioChild* ) ) );
  connect( page, SIGNAL( sig_removeChild( KivioChild* ) ), SLOT( slotRemoveChild( KivioChild* ) ) );
  connect( page, SIGNAL( sig_maxColumn( int ) ), m_pCanvas, SLOT( slotMaxColumn( int ) ) );
  connect( page, SIGNAL( sig_maxRow( int ) ), m_pCanvas, SLOT( slotMaxRow( int ) ) );
  connect( page, SIGNAL( sig_polygonInvalidated( const QPointArray& ) ),
        this, SLOT( repaintPolygon( const QPointArray& ) ) );
*/
}

void KivioView::removePage( KivioPage *_t )
{
  QString m_pageName=_t->pageName();
  m_pTabBar->removeTab( _t->pageName() );
  setActivePage( m_pDoc->map()->findPage( m_pTabBar->listshow().first() ));
}

void KivioView::removeAllPages()
{
  m_pTabBar->removeAllTabs();
  setActivePage(0L);
}

void KivioView::setActivePage( KivioPage* page )
{
  if ( page == m_pActivePage )
    return;

  m_pActivePage = page;

  m_pTabBar->setActiveTab(page->pageName());

  m_pCanvas->update();

  updateToolBars();
}

void KivioView::setActiveSpawnerSet( KivioStencilSpawnerSet *set )
{
    if( set == m_pActiveSpawnerSet )
        return;

    m_pActiveSpawnerSet = set;

//    m_pStencilStackBar->setActivePage( asdf);
}

void KivioView::slotPageRenamed( KivioPage* page, const QString& old_name )
{
  m_pTabBar->renameTab( old_name, page->pageName() );
}

void KivioView::changePage( const QString& name )
{
  if ( m_pActivePage->pageName() == name )
    return;

  KivioPage *t = m_pDoc->map()->findPage(name);
  if (!t)
  	return;

  setActivePage( t );
}

void KivioView::insertPage()
{
  KivioPage *t = m_pDoc->createPage(false);

  if( t )
      m_pDoc->addPage( t );
}

void KivioView::hidePage()
{
  if (!m_pActivePage)
    return;

  m_pTabBar->hidePage();
}

void KivioView::showPage()
{
  KivioPageShow* dlg = new KivioPageShow(this,"Page show");
  dlg->show();
}

int KivioView::leftBorder() const
{
  return 0;
}

int KivioView::rightBorder() const
{
  return 0;
}

int KivioView::topBorder() const
{
  return 0;
}

int KivioView::bottomBorder() const
{
  return 0;
}

void KivioView::paperLayoutDlg()
{
    activePage()->pagePropertiesDlg();
}

void KivioView::removePage()
{
  if ( doc()->map()->count() <= 1 ) {
    QApplication::beep();
    KMessageBox::sorry( this, i18n("You cannot delete the only page of the map."), i18n("Remove page") ); // FIXME bad english? no english!
    return;
  }
  QApplication::beep();
  int ret = KMessageBox::warningYesNo(this,i18n("You are going to remove the active page.\nDo you want to continue?"),i18n("Remove page"));

  if ( ret == 3 ) {
      KivioPage* tbl = m_pActivePage;
      doc()->map()->removePage( tbl );
      removePage(tbl);
      delete tbl;
  }
}

void KivioView::slotAddPage( KivioPage* page )
{
  addPage(page);
}

void KivioView::slotUpdateView( KivioPage* page )
{
  if ( page != m_pActivePage )
    return;

  m_pCanvas->update();
}

void KivioView::paintContent( KivioPainter& painter, const QRect& rect, bool transparent )
{
//  m_pDoc->paintContent( painter, rect, transparent, m_pActivePage );
//  temporary
  m_pCanvas->update();
}

QWidget *KivioView::canvas()
{
  return canvasWidget();
}

int KivioView::canvasXOffset() const
{
  return canvasWidget()->xOffset();
}

int KivioView::canvasYOffset() const
{
  return canvasWidget()->yOffset();
}

bool KivioView::printDlg()
{
    KMessageBox::information( this, i18n("Printing is disabled for the technology preview."),
        "Printing Disabled" );

    // FIXME: Disabled for tech-preview
/*
    QPrinter prt;
    KivioPage *pPage;
    KivioPSPrinter p;
    int i=0;
    if ( m_pActivePage && QPrintDialog::getPrinterSetup(&prt) )
    {
        p.start("test.ps", m_pDoc->map()->pageList().count() );

        // Iterate through all the pages
        // FIXME: Make it adhere to those chosen in the dialog
        pPage = m_pDoc->map()->firstPage();
        while( pPage )
        {
            p.startPage( i+1 );
            pPage->print( &p );
            p.stopPage();

            i++;
            pPage = m_pDoc->map()->nextPage();
        }

        p.stop();
//    prt.setFullPage( TRUE );
//    QPainter painter;
//    painter.begin( &prt );
//    m_pActivePage->print( painter, &prt );
//    painter.end();
  }
  return true;
*/
    return true;
}

void KivioView::viewZoom( int zoom )
{
  m_pCanvas->setZoom(zoom);
  m_viewZoom->insertItem(m_pCanvas->zoom());
}

void KivioView::canvasZoomChanged( int zoom )
{
  m_viewZoom->setEditZoom(zoom);
}

KivioPage* KivioView::activePage()
{
  return m_pActivePage;
}

void KivioView::togglePageBorders( bool b )
{
  if ( b == m_bShowPageBorders )
    return;

  m_showPageBorders->setChecked(b);
  m_bShowPageBorders = b;

  m_pCanvas->update();
}

void KivioView::togglePageMargins( bool b )
{
  if ( b == m_bShowPageMargins )
    return;

  m_showPageMargins->setChecked(b);
  m_bShowPageMargins = b;

  m_pCanvas->update();
}

void KivioView::toggleShowRulers( bool b )
{
  if ( b == m_bShowRulers )
    return;

  m_showRulers->setChecked(b);
  m_bShowRulers = b;

  m_pCanvas->toggleShowRulers(b);
}

void KivioView::toggleShowGrid( bool b )
{
  if ( b == m_bShowGrid )
    return;

  m_showGrid->setChecked(b);
  m_bShowGrid = b;

  m_pCanvas->update();
}

void KivioView::customEvent( QCustomEvent* e )
{
  if (KParts::GUIActivateEvent::test(e)) {
    viewGUIActivated(((KParts::GUIActivateEvent*)e)->activated());
  }
  KoView::customEvent(e);
}

void KivioView::addStencilSet( const QString& name )
{
/*
    AddSpawnerSetDlg *dlg = new AddSpawnerSetDlg(this, "add spawn", QString("/home/landshark/thekompany/kivio/stencils"));


    if( dlg->exec() == false )
    {
        delete dlg;
        return;
    }

    m_pDoc->addSpawnerSet( dlg->spawnerSetName() );

    delete dlg;
*/
    m_pDoc->addSpawnerSet(name);
}

void KivioView::addSpawnerToStackBar( KivioStencilSpawnerSet *pSpawner )
{
    if( !pSpawner )
    {
        debug("KivioView::addSpawnerToStackBar() - NULL pSpawner");
        return;
    }

    KivioIconView *pView = new KivioIconView();
    QObject::connect( pView, SIGNAL(createNewStencil(KivioStencilSpawner*)), this, SLOT(addStencilFromSpawner(KivioStencilSpawner*)));
    
    pView->setStencilSpawnerSet( pSpawner );

    m_pDockManager->insertStencilSet(pView, pSpawner->name());
}

void KivioView::setFGColor()
{
    KivioPage *pPage = activePage();


    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        pStencil->setFGColor( m_setFGColor->color() );
        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::setBGColor()
{
    KivioPage *pPage = activePage();


    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        pStencil->setBGColor( m_setBGColor->color() );
        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::setTextColor()
{
    KivioPage *pPage = activePage();


    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        pStencil->setTextColor( m_setTextColor->color() );
        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::setLineWidth()
{
    KivioPage *pPage = activePage();

    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        pStencil->setLineWidth( m_setLineWidth->value() );
        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::groupStencils()
{
    activePage()->groupSelectedStencils();
    m_pCanvas->repaint();
}

void KivioView::ungroupStencils()
{
    activePage()->ungroupSelectedStencils();
    m_pCanvas->repaint();
}

void KivioView::selectAllStencils()
{
    activePage()->selectAllStencils();
    m_pCanvas->repaint();
}

void KivioView::unselectAllStencils()
{
    activePage()->unselectAllStencils();
    m_pCanvas->repaint();
}

QColor KivioView::fgColor()
{
    return m_setFGColor->color();
}

QColor KivioView::bgColor()
{
    return m_setBGColor->color();
}

int KivioView::lineWidth()
{
    return (int)m_setLineWidth->value();
}


void KivioView::setFontFamily( const QString &str )
{
    QFont f;

    KivioPage *pPage = activePage();

    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        f = pStencil->textFont();
        f.setFamily( str );
        pStencil->setTextFont( f );

        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::setFontSize(const QString &str )
{
    int size = str.toInt();
    QFont f;

    KivioPage *pPage = activePage();

    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        f = pStencil->textFont();
        f.setPointSize( size );
        pStencil->setTextFont( f );

        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::toggleFontBold( bool b )
{
    QFont f;

    KivioPage *pPage = activePage();
    debug("b");
    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        debug("bold");
        f = pStencil->textFont();
        f.setBold(b);
        pStencil->setTextFont( f );

        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::toggleFontItalics( bool b )
{
    QFont f;

    KivioPage *pPage = activePage();

    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        f = pStencil->textFont();
        f.setItalic(b);
        pStencil->setTextFont( f );

        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::toggleFontUnderline( bool b)
{
    QFont f;

    KivioPage *pPage = activePage();

    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        f = pStencil->textFont();
        f.setUnderline(b);
        pStencil->setTextFont( f );

        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}



void KivioView::updateToolBars()
{
    KivioStencil *pStencil;

    pStencil = activePage()->selectedStencils()->first();
    if( !pStencil )
    {
        m_setFontFamily->setFont( "times" );
        m_setFontSize->setFontSize( 12 );
        m_setBold->setChecked( false );
        m_setItalics->setChecked( false );
        m_setUnderline->setChecked( false );
        m_setLineWidth->setValue( 1.0 );
        m_setHTextAlignment->setCurrentItem( 1 );
        m_setVTextAlignment->setCurrentItem( 1 );

        m_setStencilSize->setSize( 0.0, 0.0 );
        m_setStencilPos->setPos( 0.0, 0.0 );
    }
    else
    {
        QFont f = pStencil->textFont();

        m_setFontFamily->setFont( f.family() );
        m_setFontSize->setFontSize( f.pointSize() );
        m_setBold->setChecked( f.bold() );
        m_setItalics->setChecked( f.italic() );
        m_setUnderline->setChecked( f.underline() );

        m_setLineWidth->setValue( pStencil->lineWidth() );

        m_setFGColor->setActiveColor(pStencil->fgColor());
        m_setBGColor->setActiveColor(pStencil->bgColor());
        m_setTextColor->setActiveColor(pStencil->textColor());

        m_setHTextAlignment->setCurrentItem( pStencil->hTextAlign() );
        m_setVTextAlignment->setCurrentItem( pStencil->vTextAlign() );

        m_setStencilSize->setSize( pStencil->w(), pStencil->h() );
        m_setStencilPos->setPos( pStencil->x(), pStencil->y() );
    }
}

void KivioView::slotSetStartArrow( int i )
{
    debug("Selected start arrow %d", i );
}

void KivioView::slotSetEndArrow( int i )
{
    debug("Selected end arrow %d", i );
}

void KivioView::slotSetStartArrowSize()
{
  double w,h;
  m_setStartArrowSize->size(w,h);
  debug(QString("slotSetStartArrowSize: w=%1  h=%2").arg(w).arg(h));
}

void KivioView::slotSetEndArrowSize()
{
}

void KivioView::setHParaAlign( int i )
{
    KivioPage *pPage = activePage();

    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        pStencil->setHTextAlign(i);
        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}


void KivioView::setVParaAlign( int i )
{
    KivioPage *pPage = activePage();

    KivioStencil *pStencil = pPage->selectedStencils()->first();
    while( pStencil )
    {
        pStencil->setVTextAlign( i );
        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::bringStencilToFront()
{
    activePage()->bringToFront();

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::sendStencilToBack()
{
    activePage()->sendToBack();

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::cutStencil()
{
    activePage()->cut();

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::copyStencil()
{
    activePage()->copy();
}

void KivioView::pasteStencil()
{
    activePage()->paste();

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::slotStencilSize()
{
  double w,h;
  m_setStencilSize->size(w,h);

  KivioPage *pPage = activePage();

  KivioStencil *pStencil = pPage->selectedStencils()->first();
  if ( pStencil )
  {
      pStencil->setW(w);
      pStencil->setH(h);
  }

  m_pDoc->setModified(true);
  m_pCanvas->repaint();
}

void KivioView::slotStencilPos()
{
  double x,y;
  m_setStencilPos->pos(x,y);

  KivioPage *pPage = activePage();

  KivioStencil *pStencil = pPage->selectedStencils()->first();
  if ( pStencil )
  {
      pStencil->setX(x);
      pStencil->setY(y);
  }

  m_pDoc->setModified(true);
  m_pCanvas->repaint();
}

/**
 * When passed a spawner, this will create a new stencil at 0,0.
*/
void KivioView::addStencilFromSpawner( KivioStencilSpawner *pSpawner )
{
    KivioStencil *pStencil;
    
   
    // Allocate the new stencil and set it's default size/style
    pStencil = pSpawner->newStencil();
    
    pStencil->setPosition( 0.0f, 0.0f );

    // Only set these properties if we held ctrl down
    // FIXME: Make this happen!
//    pStencil->setFGColor( fgColor() );
//    pStencil->setBGColor( bgColor() );
//    pStencil->setLineWidth( (float)lineWidth() );

        
    // Unselect everything, then the stencil to the page, and select it
    m_pActivePage->unselectAllStencils();
    m_pActivePage->addStencil( pStencil );    
    m_pActivePage->selectStencil( pStencil );
    
    // Mark the page as modified and repaint
    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void KivioView::newView()
{
  KivioDoc* thisDocument = m_pDoc;
  KivioShell* shell = new KivioShell(thisDocument->instance());
  shell->setRootDocument(thisDocument);
  shell->show();
}

